@@ -6,7 +6,9 @@ use std::io::Cursor;
 
 #[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct SessionDescriptionSerde {
+    #[serde(rename = "type")]
     pub sdp_type: SDPType,
+
     pub sdp: String,
 }
 
@@ -45,7 +47,7 @@ mod test {
                     },
                     parsed: None,
                 },
-                r#"{"sdp_type":"Offer","sdp":"sdp"}"#,
+                r#"{"type":"offer","sdp":"sdp"}"#,
             ),
             (
                 SessionDescription {
@@ -55,7 +57,7 @@ mod test {
                     },
                     parsed: None,
                 },
-                r#"{"sdp_type":"Pranswer","sdp":"sdp"}"#,
+                r#"{"type":"pranswer","sdp":"sdp"}"#,
             ),
             (
                 SessionDescription {
@@ -65,7 +67,7 @@ mod test {
                     },
                     parsed: None,
                 },
-                r#"{"sdp_type":"Answer","sdp":"sdp"}"#,
+                r#"{"type":"answer","sdp":"sdp"}"#,
             ),
             (
                 SessionDescription {
@@ -75,7 +77,7 @@ mod test {
                     },
                     parsed: None,
                 },
-                r#"{"sdp_type":"Rollback","sdp":"sdp"}"#,
+                r#"{"type":"rollback","sdp":"sdp"}"#,
             ),
             (
                 SessionDescription {
@@ -85,7 +87,7 @@ mod test {
                     },
                     parsed: None,
                 },
-                r#"{"sdp_type":"Unspecified","sdp":"sdp"}"#,
+                r#"{"type":"Unspecified","sdp":"sdp"}"#,
             ),
         ];
 
@@ -107,7 +109,7 @@ mod test {
         m.register_default_codecs()?;
         let api = APIBuilder::new().with_media_engine(m).build();
 
-        let mut pc = api.new_peer_connection(Configuration::default()).await?;
+        let pc = api.new_peer_connection(Configuration::default()).await?;
 
         let offer = pc.create_offer(None).await?;
 
